*;;;
*;;; Universal Monitor 6800
*;;;

*        CPU     6800

* TARGET: equ     "MC6800"


*        INCLUDE "config.inc"
*;;; -*- asm -*-
*;;;
*;;; Universal Monitor 6800 config file (for SBC6800)
*;;;

*;;;
*;;; Memory
*;;;

ROM_B  equ     $E000           ; ROM begin
ENTRY  EQU     $FF80           ; Entry point

WORK_B equ     $7FC0           ; WORK area begin
STACK  equ     $7FFF           ; Stack end

* ROM_B  equ     $E000           ; ROM begin
* ENTRY  EQU     $FF80           ; Entry point

* WORK_B equ     $00C0           ; WORK area begin
* STACK  equ     $00FF           ; Stack end

BUFLEN EQU     16              ; Buffer length ( 16 or above )

*;;;
*;;; Options
*;;;

* USE_IDENT = 1                   ; CPU Identification

* USE_REGCMD = 1                  ; Register related commands

*;;;
*;;; Motorola MC6850
*;;;

*USE_DEV_6850 = 1
*        IF USE_DEV_6850
*        ;; ACIA
ACIAC  equ     $8018           ; Control / Status Register
ACIAD  equ     $8019           ; Data Register
*        ENDIF

*;;;
*;;; Motorola MC6801/3 (Embedded UART)
*;;;

*USE_DEV_6801 = 0
*        IF USE_DEV_6801
*        ;; 6801/6803 Internal
*RMCR   EQU     $0010           ; Rate and Mode Control Register
*RMCR_V EQU     $05             ; CC[1:0]=01, SS[1:0]=01
*TRCSR  EQU     $0011           ; Transmit/Receive Control and Status Register
*RDR    EQU     $0012           ; Receive Data Register
*TDR    EQU     $0013           ; Transmit Data Register
*        ENDIF

*        INCLUDE "../common.inc"
*;;; -*- asm -*-
*;;;
*;;; Common header file
*;;;

*        RELAXED ON

*;;; Constants
CR     EQU     $0D
LF     EQU     $0A
BS     EQU     $08
DEL    EQU     $7F

*;;; Functions
*low     function        x,(x & 255)
*high    function        x,(x >> 8)

*        RELAXED OFF

*        ;;
*        ;; Work Area
*        ;;

        ORG     WORK_B

INBUF  RMB      BUFLEN          ; Line input buffer
DSADDR RMB      2               ; Dump start address
DEADDR RMB      2               ; Dump end address
DSTATE RMB      1               ; Dump state
GADDR  RMB      2               ; Go address
SADDR  RMB      2               ; Set address
HEXMOD RMB      1               ; HEX file mode
RECTYP RMB      1               ; Record type
PSPEC  RMB      1               ; Processor spec.

*        IF USE_REGCMD
REGA   RMB      1               ; Accumulator A
REGB   RMB      1               ; Accumulator B
REGX   RMB      2               ; Index register X
REGSP  RMB      2               ; Stack pointer SP
REGPC  RMB      2               ; Program counter PC
REGCC  RMB      1               ; Condition code register CCR
*        ENDIF

RHVAL  RMB      2               ; RDHEX Value
DMPPT  RMB      2               ; DUMP pointer
ASCPT  RMB      2               ; ASCII pointer
CKSUM  RMB      1               ; Checksum

*;;;
*;;; ROM area
*;;;

        ORG     ROM_B

CSTART EQU *
        LDS     #STACK
        JSR     INIT

        LDX     #$0000
        STX     DSADDR
        STX     SADDR
        STX     GADDR
        LDAA    #'S'
        STAA    HEXMOD
        CLR     PSPEC
*        IF USE_REGCMD
        CLR     REGA
        CLR     REGB
        STX     REGX
        STS     REGSP
        STX     REGPC
        LDAA    #$C0
        STAA    REGCC
*        ENDIF

*        ;; Opening message
        LDX     #OPNMSG
        JSR     STROUT

*        ;; CPU identification
*        IF USE_IDENT
        LDX     #$55AA
        LDAB    #100
        LDAA    #5
        ADDA    #5
        FCB     $18             ; DAA on 6800, ABA on 6803, XGDX on 6303
        CMPA    #$55
        BEQ     ID_6301
        CMPA    #110
        BEQ     ID_6801
        CMPA    #$10
        BNE     ID_UK

        LDX     #$FFFF
        FCB     $EC,$01         ; CPX $01,X on 6800, ADX $01 on MB8861
        CPX     #$0000
        BEQ     ID_8861
*        ;; MC6800
        LDX     #IM6800
        CLRA
        BRA     IDE
*        ;; MC6801/MC6803
ID_6801 EQU *
        LDX     #IM6801
        LDAA    #$01
        BRA     IDE
*        ;; HD6301/HD6303
ID_6301 EQU *
        LDX     #IM6301
        LDAA    #$03
        BRA     IDE
*        ;; MB8861/MB8870
ID_8861 EQU *
        LDX     #IM8861
        LDAA    #$04
        BRA     IDE
*        ;; Unknown
ID_UK EQU *
        JSR     HEXOUT2
        LDX     #IMUK
        CLRA
IDE EQU *
        STAA    PSPEC
        JSR     STROUT
*        ENDIF

WSTART EQU *
        LDX     #PROMPT
        JSR     STROUT
        JSR     GETLIN
        LDX     #INBUF
        JSR     SKIPSP
        JSR     UPPER
        TSTA
        BEQ     WSTART
        CMPA    #'D'
        BNE     M00
        JMP     DUMP
M00 EQU *
        CMPA    #'G'
        BNE     M01
        JMP     GO
M01 EQU *
        CMPA    #'S'
        BNE     M02
        JMP     SETM

M02 EQU *
        CMPA    #'L'
        BNE     M03
        JMP     LOADH
M03 EQU *
        CMPA    #'P'
        BNE     M04
        JMP     SAVEH

M04 EQU *
*        IF USE_REGCMD
        CMPA    #'R'
        BNE     M05
        JMP     REG
*        ENDIF
M05 EQU *
ERR EQU *
        LDX     #ERRMSG
        JSR     STROUT
        BRA     WSTART

*;;;
*;;; Dump memory
*;;;
DUMP EQU *
        INX
        JSR     SKIPSP
        JSR     RDHEX           ; 1st arg.
        TSTB
        BNE     DP0
*        ;; No arg.
        JSR     SKIPSP
        LDAA    ,X
        BNE     ERR
        LDAA    DSADDR+1        ; DEADDR = DSADDR + 128
        ADDA    #128
        STAA    DEADDR+1
        LDAA    DSADDR
        ADCA    #0
        STAA    DEADDR
        BRA     DPM
*        ;; 1st arg. found
DP0 EQU *
        LDAA    RHVAL+1         ; DSADDR = RHVAL
        STAA    DSADDR+1
        LDAA    RHVAL
        STAA    DSADDR
        JSR     SKIPSP
        LDAA    ,X
        CMPA    #','
        BEQ     DP1
        TSTA
        BNE     ERR
*        ;; No 2nd arg.
        LDAA    DSADDR+1        ; DEADDR = DSADDR + 128
        ADDA    #128
        STAA    DEADDR+1
        LDAA    DSADDR
        ADCA    #0
        STAA    DEADDR
        BRA     DPM
*        ;;
DP1 EQU *
        INX
        JSR     SKIPSP
        JSR     RDHEX
        JSR     SKIPSP
        TSTB
        BEQ     ERR
        TST     ,X
        BNE     ERR
        LDX     RHVAL
        INX
        STX     DEADDR
*        ;; DUMP main
DPM EQU *
        LDAA    DSADDR+1
        ANDA    #$F0
        STAA    DMPPT+1
        LDAA    DSADDR
        STAA    DMPPT
        CLR     DSTATE
DPM0 EQU *
        BSR     DPL
*        ;; DMPPT already incremented during DPB
*        ;; Do not need to +16 here
        JSR     CONST
        BNE     DPM1
        LDAA    DSTATE
        CMPA    #2
        BCS     DPM0
        LDAA    DEADDR+1
        STAA    DSADDR+1
        LDAA    DEADDR
        STAA    DSADDR
        JMP     WSTART
DPM1 EQU *
        LDAA    DMPPT+1
        STAA    DSADDR+1
        LDAA    DMPPT
        STAA    DSADDR
        JSR     CONIN
        JMP     WSTART

*        ;; Dump line
DPL EQU *
        LDAA    DMPPT
        LDAB    DMPPT+1
        JSR     HEXOUT4
        LDX     #DSEP0
        JSR     STROUT
        LDX     #INBUF
        STX     ASCPT
        LDAB    #16
DPL0 EQU *
        BSR     DPB
        DECB
        BNE     DPL0

        LDX     #DSEP1
        JSR     STROUT

*        ;; Print ASCII area
        LDX     #INBUF
        LDAB    #16
DPL1 EQU *
        LDAA    ,X
        INX
        CMPA    #' '
        BCS     DPL2
        CMPA    #$7F
        BCC     DPL2
        JSR     CONOUT
        BRA     DPL3
DPL2 EQU *
        LDAA    #'.'
        JSR     CONOUT
DPL3 EQU *
        DECB
        BNE     DPL1
        JMP     CRLF

*        ;; Dump byte
DPB EQU *
        LDAA    #' '
        JSR     CONOUT
        LDAA    DSTATE
        BNE     DPB2
*        ;; Dump state 0
        LDAA    DSADDR
        CMPA    DMPPT
        BNE     DPB0
        LDAA    DSADDR+1
        CMPA    DMPPT+1
        BEQ     DPB1
*        ;; Still 0 or 2
DPB0 EQU *
        LDAA    #' '
        JSR     CONOUT
        JSR     CONOUT
        LDX     ASCPT
        STAA    ,X
        INX
        STX     ASCPT
        LDX     DMPPT
        INX
        STX     DMPPT
        RTS
*        ;; Found start address
DPB1 EQU *
        LDAA    #1
        STAA    DSTATE
DPB2 EQU *
        LDAA    DSTATE
        CMPA    #1
        BNE     DPB0
*        ;; Dump state 1
        LDX     DMPPT
        LDAA    ,X
        INX
        STX     DMPPT
        LDX     ASCPT
        STAA    ,X
        INX
        STX     ASCPT
        JSR     HEXOUT2
        LDAA    DEADDR
        CMPA    DMPPT
        BNE     DPBE
        LDAA    DEADDR+1
        CMPA    DMPPT+1
        BNE     DPBE
        LDAA    #2
        STAA    DSTATE
DPBE EQU *
        RTS

*;;;
*;;; Go address
*;;;
GO EQU *
        INX
        JSR     SKIPSP
        JSR     RDHEX
        LDAA    ,X
        BEQ     GO0
        JMP     ERR
GO0 EQU *
        TSTB
        BEQ     G0
        LDX     RHVAL
*        IF USE_REGCMD
        STX     REGPC
G0 EQU *
        LDAA    REGSP
        ORAA    REGSP+1
        BEQ     GO1
        LDS     REGSP
GO1 EQU *
        LDAA    REGPC+1
        PSHA                    ; PC(L)
        LDAA    REGPC
        PSHA                    ; PC(H)
        LDAA    REGX+1
        PSHA                    ; X(L)
        LDAA    REGX
        PSHA                    ; X(H)
        LDAA    REGA
        PSHA                    ; A
        LDAA    REGB
        PSHA                    ; B
        LDAA    REGCC
        PSHA                    ; CC
        RTI
*        ELSE
*        STX     GADDR
*G0:
*        LDX     GADDR
*        JMP     ,X
*        ENDIF

*;;;
*;;; Set memory
*;;;
SETM EQU *
        INX
        JSR     SKIPSP
        JSR     RDHEX
        JSR     SKIPSP
        TST     ,X
        BNE     SMER
        TSTB
        BEQ     SM1
        LDX     RHVAL
        STX     SADDR
SM1 EQU *
        LDAA    SADDR
        LDAB    SADDR+1
        JSR     HEXOUT4
        LDX     #DSEP1
        JSR     STROUT
        LDX     SADDR
        LDAA    ,X
        JSR     HEXOUT2
        LDAA    #' '
        JSR     CONOUT
        JSR     GETLIN
        LDX     #INBUF
        JSR     SKIPSP
        LDAA    ,X
        BNE     SM2
*        ;; Empty (Increment address)
        LDX     SADDR
        INX
        STX     SADDR
        BRA     SM1
SM2 EQU *
        CMPA    #'-'
        BNE     SM3
*        ;; '-' (Decrement address)
        LDX     SADDR
        DEX
        STX     SADDR
        BRA     SM1
SM3 EQU *
        CMPA    #'.'
        BNE     SM4
*        ;; '.' (Quit)
        JMP     WSTART

SM4 EQU *
        JSR     RDHEX
        TSTB
        BEQ     SMER
        LDX     SADDR
        LDAA    RHVAL+1
        STAA    ,X
        INX
        STX     SADDR
        BRA     SM1
SMER EQU *
        JMP     ERR

*;;;
*;;; LOAD HEX file
*;;;
LOADH EQU *
        INX
        JSR     SKIPSP
        JSR     RDHEX
        JSR     SKIPSP
        TST     ,X
        BNE     SMER

*        ;;TSTB
*        ;;BNE   LH0

*        ;;CLR   RHVAL
*        ;;CLR   RHVAL+1
LH0 EQU *
        JSR     CONIN
        JSR     UPPER
        CMPA    #'S'
        BNE     LH1
        JMP     LHS0
LH1 EQU *
        CMPA    #':'
        BEQ     LHI0
LH2 EQU *
*        ;; Skip to EOL
        CMPA    #CR
        BEQ     LH0
        CMPA    #LF
        BEQ     LH0
LH3 EQU *
        JSR     CONIN
        BRA     LH2

LHI0 EQU *
        JSR     HEXIN
        STAA    CKSUM
        TAB                     ; Length

        JSR     HEXIN
        STAA    DMPPT           ; Address H
        ADDA    CKSUM
        STAA    CKSUM

        JSR     HEXIN
        STAA    DMPPT+1         ; Address L
        ADDA    CKSUM
        STAA    CKSUM

*        ;; Add offset
        LDAA    DMPPT+1
        ADDA    RHVAL+1
        STAA    DMPPT+1
        LDAA    DMPPT
        ADCA    RHVAL
        STAA    DMPPT
        LDX     DMPPT

        JSR     HEXIN
        STAA    RECTYP
        ADDA    CKSUM
        STAA    CKSUM

        TSTB
        BEQ     LHI3
LHI1 EQU *
        JSR     HEXIN
        PSHA
        ADDA    CKSUM
        STAA    CKSUM
        PULA

        TST     RECTYP
        BNE     LHI2

        STAA    ,X
        INX
LHI2 EQU *
        DECB
        BNE     LHI1
LHI3 EQU *
        JSR     HEXIN
        ADDA    CKSUM
        BNE     LHIE            ; Checksum error
        TST     RECTYP
        BEQ     LH3
        JMP     WSTART

LHIE EQU *
        LDX     #IHEMSG
        JSR     STROUT
        JMP     WSTART

LHS0 EQU *
        JSR     CONIN
        STAA    RECTYP          ; Record type

        JSR     HEXIN
        TAB                     ; B = Length+3
        STAA    CKSUM

        JSR     HEXIN
        STAA    DMPPT           ; Address H
        ADDA    CKSUM
        STAA    CKSUM

        JSR     HEXIN
        STAA    DMPPT+1         ; Address L
        ADDA    CKSUM
        STAA    CKSUM

*        ;; Add offset
        LDAA    DMPPT+1
        ADDA    RHVAL+1
        STAA    DMPPT+1
        LDAA    DMPPT
        ADCA    RHVAL
        STAA    DMPPT
        LDX     DMPPT

        SUBB    #3
        BEQ     LHS3
LHS1 EQU *
        JSR     HEXIN
        PSHA
        ADDA    CKSUM
        STAA    CKSUM           ; Checksum

        LDAA    RECTYP
        CMPA    #'1'
        BNE     LHS2

        PULA
        STAA    ,X
        INX
        BRA     LHS20

LHS2 EQU *
        PULA
LHS20 EQU *
        DECB
        BNE     LHS1
LHS3 EQU *
        JSR     HEXIN
        ADDA    CKSUM
        CMPA    #$FF
        BNE     LHSE            ; Checksum error

        LDAA    RECTYP
        CMPA    #'7'
        BEQ     LHSR
        CMPA    #'8'
        BEQ     LHSR
        CMPA    #'9'
        BEQ     LHSR
        JMP     LH3

LHSE EQU *
        LDX     #SHEMSG
        JSR     STROUT
LHSR EQU *
        JMP     WSTART

*;;;
*;;; SAVE HEX file
*;;;
SAVEH EQU *
        INX
        LDAA    ,X
        JSR     UPPER
        CMPA    #'I'
        BEQ     SH0
        CMPA    #'S'
        BNE     SH1
SH0 EQU *
        INX
        STAA    HEXMOD
SH1 EQU *
        JSR     SKIPSP
        JSR     RDHEX
        TSTB
        BEQ     SHE
        LDAA    RHVAL
        STAA    ASCPT
        LDAA    RHVAL+1
        STAA    ASCPT+1         ; (ASCPT) = Start address
        JSR     SKIPSP
        LDAA    ,X
        CMPA    #','
        BNE     SHE
        INX
        JSR     SKIPSP
        JSR     RDHEX           ; (RHVAL) = End address
        TSTB
        BEQ     SHE
        JSR     SKIPSP
        TST     ,X
        BEQ     SH2
SHE EQU *
        JMP     ERR

SH2 EQU *
        LDX     RHVAL
        INX
        STX     RHVAL
        LDAA    RHVAL+1
        SUBA    ASCPT+1
        STAA    RHVAL+1
        LDAA    RHVAL
        SBCA    ASCPT
        STAA    RHVAL           ; (RHVAL) = Length
        LDX     ASCPT           ; IX = Start address
SH3 EQU *
        BSR     SHLZ
        LDAA    RHVAL
        ORAA    RHVAL+1
        BNE     SH3

        LDAA    HEXMOD
        CMPA    #'I'
        BNE     SH4
*        ;; End record for Intel HEX
        LDX     #IHEXER
        JSR     STROUT
        JMP     WSTART
SH4 EQU *
*        ;; End record for Motorola S record
        LDX     #SRECER
        JSR     STROUT
        JMP     WSTART

SHLZ EQU *
        LDAB    #16
        TST     RHVAL
        BNE     SHL0
        LDAA    RHVAL+1         ; Length L
        CBA                     ; (A - B)
        BCC     SHL0
        TAB                     ; B = A
SHL0 EQU *
        LDAA    RHVAL+1
        SBA
        STAA    RHVAL+1
        LDAA    RHVAL
        SBCA    #0
        STAA    RHVAL

        LDAA    HEXMOD
        CMPA    #'I'
        BNE     SHLS

SHLI EQU *
*        ;; Intel HEX
        LDAA    #':'
        JSR     CONOUT

        TBA
        JSR     HEXOUT2         ; Length
        STAB    CKSUM           ; Checksum

        STX     ASCPT
        LDAA    ASCPT           ; Address H
        JSR     HEXOUT2
        LDAA    ASCPT
        ADDA    CKSUM
        STAA    CKSUM

        LDAA    ASCPT+1         ; Address L
        JSR     HEXOUT2
        LDAA    ASCPT+1
        ADDA    CKSUM
        STAA    CKSUM

        CLRA
        JSR     HEXOUT2         ; Record type
SHLI0 EQU *
        LDAA    ,X
        PSHA
        JSR     HEXOUT2         ; Data
        PULA
        ADDA    CKSUM
        STAA    CKSUM

        INX
        DECB
        BNE     SHLI0

        LDAA    CKSUM
        NEGA
        JSR     HEXOUT2
        JMP     CRLF

SHLS EQU *
*        ;; Motorola S record
        LDAA    #'S'
        JSR     CONOUT
        LDAA    #'1'
        JSR     CONOUT

        TBA
        ADDA    #2+1            ; DataLength + Addr(2) + Sum(1)
        STAA    CKSUM
        JSR     HEXOUT2

        STX     ASCPT
        LDAA    ASCPT
        JSR     HEXOUT2         ; Address H
        LDAA    ASCPT
        ADDA    CKSUM
        STAA    CKSUM

        LDAA    ASCPT+1
        JSR     HEXOUT2         ; Address L
        LDAA    ASCPT+1
        ADDA    CKSUM
        STAA    CKSUM
SHLS0 EQU *
        LDAA    ,X
        PSHA
        JSR     HEXOUT2         ; Data
        PULA
        ADDA    CKSUM
        STAA    CKSUM

        INX
        DECB
        BNE     SHLS0

        LDAA    CKSUM
        COMA
        JSR     HEXOUT2         ; Checksum
        JMP     CRLF

*;;;
*;;; Register
*;;;
*        IF USE_REGCMD
REG EQU *
        INX
        JSR     SKIPSP
        JSR     UPPER
        TSTA
        BNE     RG0
        JSR     RDUMP
        JMP     WSTART

RG0 EQU *
        STX     DMPPT
        LDX     #RNTAB
RG1 EQU *
        TST     ,X
        BEQ     RGE
        CMPA    ,X
        BEQ     RG2
        INX
        INX
        INX
        INX
        INX
        INX
        BRA     RG1
RG2 EQU *
        STX     ASCPT
        LDX     DMPPT
        INX
        JSR     SKIPSP
        TST     ,X
        BNE     RGE
        LDX     ASCPT
        LDX     4,X
        JSR     STROUT
        LDAA    #'='
        JSR     CONOUT
        LDX     ASCPT
        LDAB    1,X
        LDX     2,X
        CMPB    #1
        BNE     RG3
*        ;; 8 bit register
        LDAA    ,X
        JSR     HEXOUT2
        BRA     RG4

RG3 EQU *
*        ;; 16 bit register
        LDAA    ,X
        JSR     HEXOUT2
        LDAA    1,X
        JSR     HEXOUT2
RG4 EQU *
        LDAA    #' '
        JSR     CONOUT
        JSR     GETLIN
        LDX     #INBUF
        JSR     SKIPSP
        LDAA    ,X
        BEQ     RGR
        JSR     RDHEX
        TSTB
        BEQ     RGE
        LDX     ASCPT
        LDAB    1,X
        LDX     2,X
        CMPB    #1
        BNE     RG5
*        ;; 8 bit register
        LDAA    RHVAL+1
        STAA    ,X
        BRA     RG6

RG5 EQU *
*        ;; 16 bit register
        LDAA    RHVAL
        STAA    ,X
        LDAA    RHVAL+1
        STAA    1,X
RG6 EQU *
RGR EQU *
        JMP     WSTART

RGE EQU *
        JMP     ERR

RDUMP EQU *
        LDX     #RDSA
        JSR     STROUT
        LDAA    REGA
        JSR     HEXOUT2

        LDX     #RDSB
        JSR     STROUT
        LDAA    REGB
        JSR     HEXOUT2

        LDX     #RDSX
        JSR     STROUT
        LDAA    REGX
        LDAB    REGX+1
        JSR     HEXOUT4

        LDX     #RDSSP
        JSR     STROUT
        LDAA    REGSP
        LDAB    REGSP+1
        JSR     HEXOUT4

        LDX     #RDSPC
        JSR     STROUT
        LDAA    REGPC
        LDAB    REGPC+1
        JSR     HEXOUT4

        LDX     #RDSCC
        JSR     STROUT
        LDAA    REGCC
        JSR     HEXOUT2

        JMP     CRLF
*        ENDIF

*;;;
*;;; Other support routines
*;;;

STROUT EQU *
        LDAA    0,X
        BEQ     STROE
        JSR     CONOUT
        INX
        BRA     STROUT
STROE EQU *
        RTS

HEXOUT4 EQU *
        BSR     HEXOUT2
        TBA
HEXOUT2 EQU *
        PSHA
        LSRA
        LSRA
        LSRA
        LSRA
        BSR     HEXOUT1
        PULA
HEXOUT1 EQU *
        ANDA    #$0F
        ADDA    #'0'
        CMPA    #'9'+1
        BCS     HEXOUTE
        ADDA    #'A'-'9'-1
HEXOUTE EQU *
        JMP     CONOUT

HEXIN EQU *
        CLRA
        BSR     HI0
        ASLA
        ASLA
        ASLA
        ASLA
HI0 EQU *
        PSHB
        TAB
        JSR     CONIN
        JSR     UPPER
        CMPA    #'0'
        BCS     HIR
        CMPA    #'9'+1
        BCS     HI1
        CMPA    #'A'
        BCS     HIR
        CMPA    #'F'+1
        BCC     HIR
        SUBA    #'A'-'9'-1
HI1 EQU *
        SUBA    #'0'
        ABA
HIR EQU *
        PULB
        RTS

CRLF EQU *
        LDAA    #CR
        JSR     CONOUT
        LDAA    #LF
        JMP     CONOUT

GETLIN EQU *
        LDX     #INBUF
        CLRB
GL0 EQU *
        JSR     CONIN
        CMPA    #CR
        BEQ     GLE
        CMPA    #LF
        BEQ     GLE
        CMPA    #BS
        BEQ     GLB
        CMPA    #DEL
        BEQ     GLB
        CMPA    #' '
        BCS     GL0
        CMPA    #$80
        BCC     GL0
        CMPB    #BUFLEN-1
        BCC     GL0             ; Too long
        INCB
        STAA    0,X
        INX
        JSR     CONOUT
        BRA     GL0

GLB EQU *
        TSTB
        BEQ     GL0
        DECB
        DEX
        LDAA    #BS
        JSR     CONOUT
        LDAA    #' '
        JSR     CONOUT
        LDAA    #BS
        JSR     CONOUT
        BRA     GL0

GLE EQU *
        BSR     CRLF
        CLR     0,X
        RTS

SKIPSP EQU *
        LDAA    0,X
        CMPA    #' '
        BNE     SSE
        INX
        BRA     SKIPSP

SSE EQU *
        RTS

UPPER EQU *
        CMPA    #'a'
        BCS     UPE
        CMPA    #'z'+1
        BCC     UPE
        ADDA    #'A'-'a'
UPE EQU *
        RTS

RDHEX EQU *
        CLRB
        CLR     RHVAL
        CLR     RHVAL+1
RH0 EQU *
        LDAA    ,X
        BSR     UPPER
        CMPA    #'0'
        BCS     RHE
        CMPA    #'9'+1
        BCS     RH1
        CMPA    #'A'
        BCS     RHE
        CMPA    #'F'+1
        BCC     RHE
        SUBA    #'A'-'9'-1
RH1 EQU *
        SUBA    #'0'
        ROLA
        ROLA
        ROLA
        ROLA
        ROLA
        ROL     RHVAL+1
        ROL     RHVAL
        ROLA
        ROL     RHVAL+1
        ROL     RHVAL
        ROLA
        ROL     RHVAL+1
        ROL     RHVAL
        ROLA
        ROL     RHVAL+1
        ROL     RHVAL
        INX
        INCB
        BRA     RH0
RHE EQU *
        RTS

*;;;
*;;; Interrupt handler
*;;;
SWIH EQU *
*        IF USE_REGCMD
        LDX     #SWIMSG
        JSR     STROUT

        PULA                    ; CCR
        STAA    REGCC
        PULA                    ; B
        STAA    REGB
        PULA                    ; A
        STAA    REGA
        PULA                    ; X(H)
        STAA    REGX
        PULA                    ; X(L)
        STAA    REGX+1
        PULA                    ; PC(H)
        PULB                    ; PC(L)
        SUBB    #1
        STAB    REGPC+1
        SBCA    #0
        STAA    REGPC
        STS     REGSP
        JSR     RDUMP
        JMP     WSTART
*        ELSE
*        ;; Dummy
*        RTI
*        ENDIF

DUMMYH EQU *
        RTI

*;;;
*;;; Strings
*;;;

OPNMSG EQU *
        FCB      CR,LF
        FCC      "Universal Monitor 6800"
        FCB      CR,LF,$00

PROMPT EQU *
        FCC      "] "
        FCB      $00

IHEMSG EQU *
        FCC      "Error ihex"
        FCB      CR,LF,$00

SHEMSG EQU *
        FCC      "Error srec"
        FCB      CR,LF,$00

ERRMSG EQU *
        FCC      "Error"
        FCB      CR,LF,$00

DSEP0 EQU *
        FCC      " :"
        FCB      $00
DSEP1 EQU *
        FCC      " : "
        FCB      $00
IHEXER EQU *
        FCC      ":00000001FF"
        FCB      CR,LF,$00
SRECER EQU *
        FCC      "S9030000FC"
        FCB      CR,LF,$00

*        IF USE_IDENT
IMUK   FCC     "Unknown"
        FCB      CR,LF,$00
IM6800 FCC     "MC6800"
        FCB      CR,LF,$00
IM6801 FCC     "MC6801"
        FCB      CR,LF,$00
IM6301 FCC     "HD6301"
        FCB      CR,LF,$00
IM8861 FCC     "MB8861"
        FCB      CR,LF,$00
*        ENDIF

*        IF USE_REGCMD

SWIMSG FCC      "SWI"
        FCB      CR,LF,$00
RDSA   FCC      "A="
        FCB      $00
RDSB   FCC      " B="
        FCB      $00
RDSX   FCC      " X="
        FCB      $00
RDSSP  FCC      " SP="
        FCB      $00
RDSPC  FCC      " PC="
        FCB      $00
RDSCC  FCC      " CCR="
        FCB      $00

RNTAB EQU *
        FCB      'A',1
        FDB    REGA,RNA
        FCB      'B',1
        FDB    REGB,RNB
        FCB      'X',2
        FDB    REGX,RNX
        FCB      'S',2
        FDB    REGSP,RNSP
        FCB      'P',2
        FDB    REGSP,RNPC
        FCB      'C',1
        FDB    REGCC,RNCC

        FCB      $00,0           ; End mark
        FDB    0

RNA    FCC      "A"
        FCB      $00
RNB    FCC      "B"
        FCB      $00
RNX    FCC      "X"
        FCB      $00
RNSP   FCC      "SP"
        FCB      $00
RNPC   FCC      "PC"
        FCB      $00
RNCC   FCC      "CCR"
        FCB      $00

*        ENDIF

*        IF USE_DEV_6850
*        INCLUDE "dev/unimon_dev_6850.asm"

INIT EQU *
        LDAA    #$03
        STAA    ACIAC
        NOP
        NOP
        LDAA    #$15            ; x16 8bit N 1
        STAA    ACIAC

        RTS

CONIN EQU *
        LDAA    ACIAC
        ANDA    #$01
        BEQ     CONIN
        LDAA    ACIAD

        RTS

CONST EQU *
        LDAA    ACIAC
        ANDA    #$01

        RTS

CONOUT EQU *
        PSHA
CO0 EQU *
        LDAA    ACIAC
        ANDA    #$02
        BEQ     CO0
        PULA
        STAA    ACIAD

        RTS

*        ENDIF

*        IF USE_DEV_6801
*        INCLUDE "dev/unimon_dev_6801.asm"
*        ENDIF

*       ALTAIR MONTOR COMPATIBLE ENTRY

        ORG     $FF00
*       INPUT ONE CHARACTER ACC B
INCH    BSR POLCAT
        BCC INCH
        LDAB ACIAD ; GET CHAR
        RTS

        ORG     $FF24
POLCAT  LDAB ACIAC ; ACIA STATUS TO B
        ASRB ; ROTATE RDRF BIT INTO CARRY
        RTS ; RETURN

        ORG     $FF70
*       OUTCH OUTPUT CHARACTER ACC A
OUTCH2  BSR POLCAT
        ASRB
        BCC OUTCH2
        ANDA #$7F ; Strip parity bit
        STAA ACIAD ; OUTPUT
        RTS

*        ;;
*        ;; Entry point
*        ;;

        ORG     ENTRY+0         ; Cold start
E_CSTART EQU *
        FCB     $8C             ; CPX (Skip 2 bytes)
OUTCH   BRA     OUTCH2
        JMP     CSTART

        ORG     ENTRY+8         ; Warm start
E_WSTART EQU *
        JMP     WSTART

        ORG     ENTRY+16        ; Console output
E_CONOUT EQU *
        JMP     CONOUT

        ORG     ENTRY+24        ; (Console) String output
E_STROUT EQU *
        JMP     STROUT

        ORG     ENTRY+32        ; Console input
E_CONIN EQU *
        JMP     CONIN

        ORG     ENTRY+40        ; Console status
E_CONST EQU *
        JMP     CONST

*        ;;
*        ;; Vector Area
*        ;;

        ORG     $fff8
        FDB    DUMMYH          ; IRQ
        FDB    SWIH            ; SWI
        FDB    DUMMYH          ; NMI
        FDB    CSTART          ; RESET

        END
*        ;;
*        ;; Work Area
*        ;;

        ORG     WORK_B

INBUF  RMB      BUFLEN          ; Line input buffer
DSADDR RMB      2               ; Dump start address
DEADDR RMB      2               ; Dump end address
DSTATE RMB      1               ; Dump state
GADDR  RMB      2               ; Go address
SADDR  RMB      2               ; Set address
HEXMOD RMB      1               ; HEX file mode
RECTYP RMB      1               ; Record type
PSPEC  RMB      1               ; Processor spec.

*        IF USE_REGCMD
REGA   RMB      1               ; Accumulator A
REGB   RMB      1               ; Accumulator B
REGX   RMB      2               ; Index register X
REGSP  RMB      2               ; Stack pointer SP
REGPC  RMB      2               ; Program counter PC
REGCC  RMB      1               ; Condition code register CCR
*        ENDIF

RHVAL  RMB      2               ; RDHEX Value
DMPPT  RMB      2               ; DUMP pointer
ASCPT  RMB      2               ; ASCII pointer
CKSUM  RMB      1               ; Checksum

        END
